//	Initialize the copyWidget object
prism.run([
    "$filter",
    "plugin-Forecasting.services.bindOnce",
    "plugin-Forecasting.services.regressionFunc",
    function($filter, $bindOnce, $regressionFunc) {

        //////////////////////////
        //	Plugin properties	//
        //////////////////////////

        //	Regression options
        mod.none = "none";

        let regressionOptions = {
            None: mod.none,
            Linear: "linear",
            Exponential: "exponential",
            Logarithmic: "logarithmic"};

        mod.regressionProperty = "regressionType";

        //	Prediction methods for how many
        let calendarOption = "calendar end";

        // Adding the value for backwards computability
        mod.predictionMethods = [
            {
                label: "None",
                value: mod.none
            },
            {
                label: "Calendar end",
                value: calendarOption
            },
            {
                label: "Next 3",
                value: "Next 3",
                numPeriods: 3
            },
            {
                label: "Next 6",
                value: "Next 6",
                numPeriods: 6
            },
            {
                label: "Next 12",
                value: "Next 12",
                numPeriods: 12
            },
        ];

        //	Default settings
        mod.defaultSettings = {
            enabled : false,
            predictionMethod: calendarOption,
            periodsAdded:false,
        };

        //	Get the date formatter
        mod.dateFormatter = $filter("levelDate");

        function onWidgetprocessresult(widget, result) {
            $regressionFunc.startRegression(result.widget, result)
        }

        function onDashboardLoaded(e, {dashboard}) {
            $bindOnce(dashboard, "widgetprocessresult", onWidgetprocessresult);
        }

        // Registering dashboard/ widget creation in order to perform drilling
        prism.on("dashboardloaded", onDashboardLoaded);

        //	Function to create the axis menu items
        let predictionOptionItems = (settings, widget) => {
            //	Create an array to hold the list
            let items = [];

            //	Get the setting for this
            let selectedMethod = settings.predictionMethod;

            //	Function the runs when an item is picked
            let itemPicked = function(){

                //	Update the axis setting
                this.settings.predictionMethod = this.value;
                widget.options.regression = this.settings;

                //	Redraw the widget
                widget.redraw();
            };

            //	Add all prediction methods
            mod.predictionMethods.forEach((method) => {

                //	Get the option as a string
                let option = method.label;

                //	Create the menu option
                let item = {
                    caption: option,
                    checked: (method.value === selectedMethod),
                    size: "xl",
                    type: "check",
                    widget: widget,
                    settings: settings,
                    value: method.value,
                    execute: itemPicked
                };

                //	Add to the list
                items.push(item);
            });

            //	Return the list of items
            return items;
        };

        //	Create Options for the quadrant analysis
        prism.on("beforemenu", function (e, args) {

            let widget = $$get(e,"currentScope.widget") || $$get(args, "settings.widget");

            if (!defined(widget) || prism.$ngscope.appstate !== "widget"){
                return;
            }

            const widgetIsSupported = $regressionFunc.widgetIsSupported;

            // Regression line does not supported this widgets types
            const isNotSupportedWidgetSubType = widgetIsSupported.widgetSubType(widget.subtype);
            const isNotSupportedWidgetType = !widgetIsSupported.widgetType(widget.type);

            if (isNotSupportedWidgetSubType || isNotSupportedWidgetType){
                return;
            }

            //	Define the menu label
            let headerMenuCaption = "Regression Line";

            //	Can we show the regression options?
            // Yes if there are enabled items in the panel and
            let addMeasureMenuItem = $regressionFunc.canEnableRegression(e, args, headerMenuCaption, "measure"),
                addSettingsMenuItem = $regressionFunc.canEnableRegression(e, args, headerMenuCaption,"settings");

            //	Add the option to enable the regression line on metadata item
            if (addMeasureMenuItem) {

                //	Look for a saved regression property
                let savedRegressionType = args.settings.item[mod.regressionProperty] ?
                    args.settings.item[mod.regressionProperty] : mod.none;

                //	Function to create the measure sub-menu items
                let regressionTypeItems = (panelItem) => {
                    //	Create an array to hold the list
                    let items = [];
                        //	Get the setting for this
                        //selectedType = savedRegressionType;

                    //	Loop through each regression option and add sub-menu item
                    for (let option in regressionOptions) {

                        if (!regressionOptions.hasOwnProperty(option)){
                            return;
                        }

                        //	Create the menu option
                        let item = {
                            caption: option,
                            // two conditions for backwards compatibility
                            checked: (regressionOptions[option] === savedRegressionType) || option === savedRegressionType,
                            size: "xl",
                            type: "check",
                            panelItem: panelItem,
                            execute: () => {
                                // Update the axis setting
                                panelItem[mod.regressionProperty] = regressionOptions[option];
                                // Redraw the widget
                                widget.redraw();
                            }
                        };

                        //	Add to the list
                        items.push(item);
                    }

                    //	Return the list of items
                    return items;
                };

                //	Create menu for picking the regression measure
                let regressionTypes = {
                    caption: "Regression",
                    items: regressionTypeItems(args.settings.item)
                };

                //	Add options to the menu
                args.settings.items.push(regressionTypes);
            }

            //	Add the options in the settings menu
            if (addSettingsMenuItem) {

                //	Get saved settings
                let settings = widget.options.regression;

                //	Create defaults, if not defined
                if (!defined(settings)){
                    settings = mod.defaultSettings;
                }

                //	Create header
                let header = {
                    caption: headerMenuCaption,
                    type: "header",
                    disabled: false
                };

                //	Create menu for picking the regression measure
                let precitionMenuOptions = {
                    caption: "Prediction Options",
                    items: predictionOptionItems(settings, widget)
                };

                //	Add options to the menu
                args.settings.items.push(header);
                args.settings.items.push(precitionMenuOptions);
            }
        });
    }]);